@@ -3,7 +3,7 @@ from .cast import cast_str, cast_int, cast_datetime, cast_date
 from .date import now, get_date, get_datetime, date_range
 from .excel import update_gspread, read_gspread, clear_gspead, to_excel_date
 from .logs import CustomLogger, dumps_map, unraw
-from .map import chain_exists, filter_data, df_exist
+from .map import chain_exists, filter_data, re_get, df_exist
 from .map import exists_one, cast_get, list_get
 from .parse import parse_cookies
 
@@ -59,9 +59,11 @@ HEADERS = {
 DEFAULT_LIMIT = 100
 REDIRECT_LIMIT = 10
 
+REDIRECT_MSG = lambda operation: f"{operation} operation is redirecting"
+
 GCLOUD_ACCOUNT = "env/gcloud.json"
 
-parse_path = lambda url: re.sub(urlparse(url).path+'$','',url)
+parse_path = lambda url: re_get(f"(.*)(?={urlparse(url).path})", url) if urlparse(url).path else url
 
 encrypt = lambda s=str(), count=0, *args: encrypt(
     base64.b64encode(str(s).encode("utf-8")).decode("utf-8"), count-1) if count else s
@@ -160,8 +162,9 @@ class Spider(CustomDict):
 
     def __init__(self, operation=str(), filter: Optional[List[str]]=list(), filterArgs: Callable[[],Dict]=None,
                 logName=str(), logLevel="WARN", logFile=str(), logErrors=False, errorArgs=list(), errorKwargs=list(),
-                delay=1., numTasks=100, maxLoops=1, progress=tqdm, debug=False,
-                queryKey=str(), querySheet=str(), queryFields: Optional[List[str]]=list(),
+                delay=1., numTasks=100, maxLoops=1, progress=tqdm, debug=False, queryKey=str(), querySheet=str(),
+                queryFields: Optional[List[str]]=list(), queryString: Optional[List[str]]=list(),
+                queryRename: Optional[Dict[str,str]]=dict(), account: Optional[Dict]=dict(),
                 apiRedirect=False, redirectUnit=1, redirectErrors=False, localSave=True, extraSave=False, **kwargs):
         self.operation = self.operation
         self.initTime = now()
@@ -179,22 +182,33 @@ class Spider(CustomDict):
         self.maxLoops = maxLoops
         self.tqdm = progress
         self.debug = debug
+        self.account = account
         self.apiRedirect = apiRedirect
         self.redirectUnit = redirectUnit
         self.redirectErrors = redirectErrors
         self.localSave = localSave
         self.extraSave = extraSave
-        if filterArgs: self.update(filterArgs(**kwargs))
+        self.update(filterArgs(**kwargs) if filterArgs else kwargs)
         if queryKey and querySheet:
-            self.read_query(queryKey, querySheet, queryFields)
+            self.read_query(queryKey, querySheet, queryFields, queryString, queryRename, account)
 
     ###################################################################
     ######################### Session Managers ########################
     ###################################################################
 
+    def requests_task(func):
+        @functools.wraps(func)
+        def wrapper(self: Spider, *args, **kwargs):
+            if not (args or kwargs): kwargs = self.__dict__.copy()
+            results = func(self, *args, **kwargs)
+            self.upload_data(results, **kwargs)
+            return results
+        return wrapper
+
     def requests_session(func):
         @functools.wraps(func)
         def wrapper(self: Spider, *args, **kwargs):
+            if not (args or kwargs): kwargs = self.__dict__.copy()
             with requests.Session() as session:
                 results = func(self, *args, session=session, **kwargs)
             time.sleep(.25)
@@ -260,7 +274,7 @@ class Spider(CustomDict):
         self.logger.debug(log_messages(headers=headers, json=self.logJson))
         with session.get(url, headers=headers) as response:
             self.logger.info(log_response(response, url=url, query=query))
-            results = json.loads(response.text())
+            results = json.loads(response.text)
         self.logger.info(log_results(results, query=query))
         return results
 
@@ -268,8 +282,9 @@ class Spider(CustomDict):
     ############################# Methods #############################
     ###################################################################
 
-    def read_query(self, queryKey: str, querySheet: str, queryFields: Union[List,str], **kwargs):
-        data = read_gspread(queryKey, querySheet, numericise=False)
+    def read_query(self, queryKey: str, querySheet: str, queryFields: Union[List,str],
+                    queryString=list(), queryRename=dict(), account=dict(), **kwargs):
+        data = read_gspread(queryKey, querySheet, account, numericise_ignore=queryString, rename=queryRename)
         self.logger.info(log_table(data, json=self.logJson))
         if isinstance(queryFields, str): queryFields = queryFields.split(',')
         for queryName in queryFields:
@@ -328,14 +343,12 @@ class Spider(CustomDict):
                         date_cols: Optional[List[str]]=list(), datetime_cols: Optional[List[str]]=list(),
                         str_cols: Optional[List[str]]=list(), account: Optional[Dict]=dict(), **kwargs) -> pd.DataFrame:
         if not (key and sheet): return pd.DataFrame()
-        history = read_gspread(key, sheet, account).rename(columns=rename)
+        history = read_gspread(key, sheet, account, numericise_ignore=str_cols, rename=rename)
         self.logger.info(log_table(history, key=key, sheet=sheet, json=self.logJson))
         for column in date_cols:
             if column in history: history[column] = history[column].apply(cast_date)
         for column in datetime_cols:
             if column in history: history[column] = history[column].apply(cast_datetime)
-        for column in str_cols:
-            if column in history: history[column] = history[column].apply(cast_str)
         return history
 
     def map_gs_history(self, data: pd.DataFrame, history: pd.DataFrame, **kwargs) -> pd.DataFrame:
@@ -416,9 +429,20 @@ class AsyncSpider(Spider):
     ########################## Async Managers #########################
     ###################################################################
 
+    def asyncio_task(func):
+        @functools.wraps(func)
+        async def wrapper(self: AsyncSpider, *args, **kwargs):
+            if not (args or kwargs): kwargs = self.__dict__.copy()
+            semaphore = self.asyncio_semaphore(**kwargs)
+            results = await func(self, *args, semaphore=semaphore, **kwargs)
+            self.upload_data(results, **kwargs)
+            return results
+        return wrapper
+
     def asyncio_session(func):
         @functools.wraps(func)
         async def wrapper(self: AsyncSpider, *args, **kwargs):
+            if not (args or kwargs): kwargs = self.__dict__.copy()
             semaphore = self.asyncio_semaphore(**kwargs)
             async with aiohttp.ClientSession() as session:
                 results = await func(self, *args, session=session, semaphore=semaphore, **kwargs)
@@ -487,7 +511,7 @@ class AsyncSpider(Spider):
 
     @Spider.gcloud_authorized
     async def redirect(self, startDate: dt.date, endDate: dt.date, message=str(), **kwargs) -> List[Dict]:
-        message = message if message else f"{self.operation} operation is redirecting"
+        message = message if message else REDIRECT_MSG(self.operation)
         return chain_exists(await self.tqdm.gather(
             *[self.fetch_redirect(startDate=date, endDate=date, **kwargs)
                 for date in date_range(startDate, endDate)], desc=message))
@@ -605,15 +629,17 @@ class EncryptedSpider(Spider):
     extraKeys = list()
     where = str()
 
-    def __init__(self, operation=str(), select: Optional[List[str]]=list(), extraSelect: Optional[List[str]]=list(),
-                configs: Optional[List[Dict]]=list(), logFile=str(), logLevel="WARN", logErrors=False,
-                errorArgs=list(), errorKwargs=list(), delay=1., numTasks=100, maxLoops=1, progress=tqdm,
-                debug=False, confirm=True, queryKey=str(), querySheet=str(), queryFields: Optional[List[str]]=list(),
+    def __init__(self, operation=str(), filter: Optional[List[str]]=list(), filterArgs: Callable[[],Dict]=None,
+                logName=str(), logLevel="WARN", logFile=str(), logErrors=False, errorArgs=list(), errorKwargs=list(),
+                delay=1., numTasks=100, maxLoops=1, progress=tqdm, debug=False, queryKey=str(), querySheet=str(),
+                queryFields: Optional[List[str]]=list(), queryString: Optional[List[str]]=list(),
+                queryRename: Optional[Dict[str,str]]=dict(), account: Optional[Dict]=dict(),
                 apiRedirect=False, redirectUnit=1, redirectErrors=False, localSave=True, extraSave=False,
                 cookies=str(), encryptedKey=str(), **kwargs):
-        super().__init__(operation, select, extraSelect, configs, logFile, logLevel, logErrors, errorArgs, errorKwargs,
-                        delay, numTasks, maxLoops, progress, debug, confirm, queryKey, querySheet, queryFields, apiRedirect,
-                        redirectUnit, redirectErrors, localSave, extraSave, **kwargs)
+        super().__init__(operation, filter, filterArgs, logName, logLevel, logFile, logErrors, errorArgs, errorKwargs,
+                        delay, numTasks, maxLoops, progress, debug, queryKey, querySheet, queryFields,
+                        queryString, queryRename, account, apiRedirect, redirectUnit, redirectErrors,
+                        localSave, extraSave, **kwargs)
         self.cookies = cookies
         self.read_secrets(encryptedKey)
 
@@ -656,15 +682,17 @@ class EncryptedAsyncSpider(AsyncSpider, EncryptedSpider):
     extraKeys = list()
     where = str()
 
-    def __init__(self, operation=str(), select: Optional[List[str]]=list(), extraSelect: Optional[List[str]]=list(),
-                configs: Optional[List[Dict]]=list(), logFile=str(), logLevel="WARN", logErrors=False,
-                errorArgs=list(), errorKwargs=list(), delay=1., numTasks=100, maxLoops=1, progress=tqdm,
-                debug=False, confirm=True, queryKey=str(), querySheet=str(), queryFields: Optional[List[str]]=list(),
+    def __init__(self, operation=str(), filter: Optional[List[str]]=list(), filterArgs: Callable[[],Dict]=None,
+                logName=str(), logLevel="WARN", logFile=str(), logErrors=False, errorArgs=list(), errorKwargs=list(),
+                delay=1., numTasks=100, maxLoops=1, progress=tqdm, debug=False, queryKey=str(), querySheet=str(),
+                queryFields: Optional[List[str]]=list(), queryString: Optional[List[str]]=list(),
+                queryRename: Optional[Dict[str,str]]=dict(), account: Optional[Dict]=dict(),
                 apiRedirect=False, redirectUnit=1, redirectErrors=False, localSave=True, extraSave=False,
                 cookies=str(), encryptedKey=str(), **kwargs):
-        super().__init__(operation, select, extraSelect, configs, logFile, logLevel, logErrors, errorArgs, errorKwargs,
-                        delay, numTasks, maxLoops, progress, debug, confirm, queryKey, querySheet, queryFields, apiRedirect,
-                        redirectUnit, redirectErrors, localSave, extraSave, **kwargs)
+        super().__init__(operation, filter, filterArgs, logName, logLevel, logFile, logErrors, errorArgs, errorKwargs,
+                        delay, numTasks, maxLoops, progress, debug, queryKey, querySheet, queryFields,
+                        queryString, queryRename, account, apiRedirect, redirectUnit, redirectErrors,
+                        localSave, extraSave, **kwargs)
         self.cookies = cookies
         self.read_secrets(encryptedKey)
 
@@ -716,11 +744,9 @@ class Pipeline(Spider):
     returnType = "dataframe"
 
     @abstractmethod
+    @Spider.requests_task
     def crawl(self, query: List[str], **kwargs) -> Union[List[Dict],pd.DataFrame]:
-        results = self.gather(query, **kwargs)
-        results = self.map_reduce(results, **kwargs)
-        self.upload_data(results, **kwargs)
-        return results
+        return self.gather(query, **kwargs)
 
     @Spider.response_filter
     def gather(self, query: List[str], **kwargs) -> Union[List[Dict],pd.DataFrame]:
@@ -728,7 +754,7 @@ class Pipeline(Spider):
         results1 = crawler1.crawl(**crawler1.__dict__)
         crawler2 = Spider(query=query, **kwargs)
         results2 = crawler2.crawl(**crawler2.__dict__)
-        return results1 + results2
+        return self.map_reduce(results1 + results2, **kwargs)
 
     def map_reduce(self, data: Union[List[Dict],pd.DataFrame], **kwargs) -> Union[List[Dict],pd.DataFrame]:
         return data
@@ -740,11 +766,9 @@ class AsyncPipeline(AsyncSpider):
     returnType = "dataframe"
 
     @abstractmethod
+    @AsyncSpider.asyncio_task
     async def crawl(self, query: List[str], **kwargs) -> Union[List[Dict],pd.DataFrame]:
-        results = await self.gather(query, **kwargs)
-        results = self.map_reduce(results, **kwargs)
-        self.upload_data(results, **kwargs)
-        return results
+        return await self.gather(query, **kwargs)
 
     @abstractmethod
     @AsyncSpider.asyncio_filter
@@ -753,7 +777,7 @@ class AsyncPipeline(AsyncSpider):
         results1 = await crawler1.crawl(**crawler1.__dict__)
         crawler2 = AsyncSpider(query=query, **kwargs)
         results2 = await crawler2.crawl(**crawler2.__dict__)
-        return results1 + results2
+        return self.map_reduce(results1 + results2, **kwargs)
 
     def map_reduce(self, data: Union[List[Dict],pd.DataFrame], **kwargs) -> Union[List[Dict],pd.DataFrame]:
         return data
